if (!Nav) throw "Requires: [Nav]";
/**
 * Nav.Pagers
 * 
 * Nav.pagers = new Nav.Pagers({
 *		background: 'Page-wrap4',
 *		backgroundPositionStatic: '-120px top'
 * });
 * 
 * 
 * @package 	Nav
 * @depends		Nav
 */
Nav.Pagers = new Class({
	
	Extends: Options,
	
	options: {
		pagers: 					'.pager',
		pagerL: 					'Pager-left',
		pagerR: 					'Pager-right',
		background: 				'Page-wrapper',
		pagersMinWidth: 			1100,
		backgroundMinWidth: 		960,
		backgroundPositionFluid: 	'50% top',
		backgroundPositionStatic: 	'left top',
		getPagerLinks:				function(){}
	},
	
	log: $L.from('Nav.Pagers'),
	pagersShowing: true,
	
	initialize: function(options)
	{
		this.setOptions(options);
		
		this.pagers = $$(this.options.pagers);
		this.pagerL = $(this.options.pagerL);
		this.pagerR = $(this.options.pagerR);
		this.background = $(this.options.background);
		
		this.pagers.addEvent('focus', function(e){
			e.target.blur();
		});
		
		this.resizeStart();
		this.pagersStart();
	},
	
	/**
	 * Handle resizing of the browser window with regard to pagers
	 */
	resizeStart: function(){
		var onResize = function(e){
			var size = window.getSize();
			
			// Pagers
			if (size.x < this.options.pagersMinWidth){
				this.resizedPagers(false);
			} else {
				this.resizedPagers(true);
			}
			
			// Background
			if (size.x < this.options.backgroundMinWidth){
				this.resizedBackground(false);
			} else {
				this.resizedBackground(true);
			}
			
		}.bind(this);
		
		window.addEvent('resize', onResize);
		onResize();
	},
	
	/**
	 * Toggle pagers
	 * @param show
	 */
	resizedPagers: function(show){
		if (show != this.pagersShowing)
		{
			this.log.debug('Show pagers: ' + show);
			
			this.pagers.fade(show?'in':'out');
			this.pagersShowing = show;
		}
	},
	
	/**
	 * Toggle background position
	 * @param fluid
	 */
	resizedBackground: function(fluid){
		if (fluid != this.backgroundFluid)
		{
			this.log.debug('Background fluid: ' + fluid);
			
			this.background.setStyle('background-position', fluid 
					? this.options.backgroundPositionFluid
					: this.options.backgroundPositionStatic);
			
			this.backgroundFluid = fluid;
		}
	},
	
	/**
	 * Handle history, link href swapping
	 */
	pagersStart: function()
	{
		// Handle historychange events
		window.addEvent('historychange', this.historychangeHandler.bind(this));
	},
		
	historychangeHandler: function(url)
	{
		// Get the base of the URL: /rates/diving gives "rates", / gives ""
		var base = Nav.instance().getUrlBase(url);
		this.log.debug('History changed, url base: '+base);
		
		var pagerLinks = this.options.getPagerLinks(base);
		
		this.pagerL.set('href', pagerLinks.prev);
		this.pagerR.set('href', pagerLinks.next);
	}
	
});

