if (!History) throw "Requires: [History]";
/**
 * Nav
 * 
 * @package 	Nav
 * @depends		History
 */
var Nav = new Class({
	
	Implements: [Options],
	
	log: $L.from('Nav'),
	
	initialize: function(options)
	{
		this.setOptions(options);
	},
	
	/**
	 * Busy state - used for keeping track of when Nav is busy loading pages.
	 */
	_busy: false,
	
	/**
	 * Set busy state
	 * @param busy
	 */
	setBusy: function(busy)
	{
		this._busy = !! busy;
		window.fireEvent('navbusy', busy);
	},
	
	/**
	 * Get busy state
	 * @returns {Boolean}
	 */
	isBusy: function()
	{
		this.log.debug('BUSY: '+this._busy);
		return this._busy;
	},
	
	/**
	 * Execute the callback if Nav is not busy, otherwise ignore.
	 * @param callback
	 */
	ifNotBusy: function(callback)
	{
		if (this.isBusy())
		{
			this.log.info('ACTION: ignore');
		}
		else
		{
			this.log.info('ACTION: execute');
			callback();
		}
	},
	
	/**
	 * Find the wrapping anchor.
	 * If anchor is a an anchor tag then return itself otherwise check ancenstors.
	 * @param anchor
	 */
	findAnchor: function(anchor)
	{
		return anchor.get('tag') === 'a' ? anchor : anchor.getParent('a');
	},
	
	/**
	 * Converts:
	 * / to ""
	 * /rates to "rates"
	 * /rates/diving to "rates"
	 * 
	 * @param url
	 * @returns string
	 */
	getUrlBase: function(url)
	{
		return url.split('/', 2)[1];
	},
	
	/**
	 * Converts
	 * default to ""
	 * rates to "rates"
	 * rates_diving to "rates"
	 * rates/diving to "rates"
	 * 
	 * @param path
	 * @returns string
	 */
	getPathBase: function(path)
	{
		// Normalise by stripping off the / at the beginning if exists
		path = path.replace(/^\//, '') || '';
		var base = this.getUrlBase('/'+path).split('_', 1)[0];
		return base === this.options.home ? '' : base;
	}
	
});


/**
 * Singleton pattern
 * @returns Nav
 */
Nav.instance = function()
{
	if (!Nav._instance)
	{
		$L.from('Nav').debug('Creating singleton instance of Nav');
		Nav._instance = new Nav(Nav.options || {});
	}
	return Nav._instance;
};


/**
 * Constructor options
 */
Nav.options = {
	home: 			'default'
};