/**
 * Initialise a new BusyTracker every time you want to use it. This 
 * guarantees that you get a clean copy each time with no tasks in it.
 * 
 * The contructor takes a callback that is executed when all tasks
 * have completed.
 * 
 * @author Paul
 * @date 27/07/2011
 */
var BusyTracker = new Class({

	tasks: null,
	onComplete: null,
	
	/**
	 * Instantiate and supply an onComplete callback that will be called when done.
	 * @param onComplete
	 */
	initialize: function(onComplete)
	{
		this.tasks = [];
		this.onComplete = onComplete;
	},

	/**
	 * Add a task to the Busy Tracker
	 * @param task
	 * @returns self
	 */
	addTask: function(task)
	{
		var nextIndex = this.tasks.length;
		this.tasks[nextIndex] = true;
		// Execute the task and pass in a completion callback
		task(function(){
			this.tasks[nextIndex] = false;
			this._checkBusy();
		}.bind(this));
		return this;
	},
	
	_checkBusy: function()
	{
		var busy = this.tasks.some(function(item){
			return !!item;
		});
		if (!busy){
			this.onComplete();
		}
		return busy;
	}
	
});