if (!Website) throw "Requires: [host page.Website]";
/**
 * Website.DisplayBase
 * 
 * @package 	Website
 * @depends		Website
 */
Website.DisplayBase = new Class({
	
	Implements: [Options],
	
	options: {
		caret: null,
		content: null,
		baseContainerId: null,
		caretElementId: null,
		loadingElementId: null,
		backgroundId: null,
		slides: null,
		ajaxBaseUrl: null,
		idealDuration: null,
		contentSetter: function(container, html)
		{
			container.empty().set('html', html);
		}
	},
	
	mainmenu: Website.mainMenuItems,
	caret: null,
	content: null,
	
	log: $L.from('Website.DisplayBase'),
	
	initialize: function(options)
	{
		this.setOptions(options);
		
		this.caret = new Fx.Morph(this.options.caretElementId, this.options.caret);
		
		this.content = new Fx.Morph(this.options.baseContainerId, this.options.content);
		
		this.backgroundFx = new Fx.Morph(this.options.backgroundId, this.options.background);
		
		// Just in case the background didn't fade out 
		window.addEvent('basepageloaded', function(){
			$(this.options.backgroundId).fade('out');
		}.bind(this));
		
	},
	
	animateCaret: function(code)
	{
		var css = '.caret-'+code;
		this.log.debug('Set css class to: '+css);
		this.caret.start(css);
	},
	
	/**
	 * Organiser
	 * @param request
	 * @param container
	 * @param level
	 * @param url
	 * @param hijack
	 * @param code
	 * @param busyTracker
	 */
	swapPage: function(request, container, mappedUrl, hijack, code, busyTracker)
	{
		// This has already been done in Website.Nav but no harm in making sure.
		this.animateCaret(code);
		
		/**
		 * Set up a BusyTracker to wait for the latter of the four events:
		 * - ...
		 */
		busyTracker.addTask(function(onComplete){
			// Start fading out immediately, ends once fade is done and has no further business
			this.fadeOutAndSlidePage(code, onComplete);
		}.bind(this));
		
		busyTracker.addTask(function(onComplete){
			this.fadeInBackground(code, onComplete);
		}.bind(this));
		
		busyTracker.addTask(function(onComplete){
			this.load(request, container, mappedUrl, hijack, onComplete);
		}.bind(this));
	},
	
	
	/**
	 * Tasks
	 */
	fadeOutAndSlidePage: function(code, onComplete)
	{
		try
		{
			this.log.debug('Fading out page.');
			
			// Chain when the effect completes or is cancelled
			this.content.removeEvents('chainComplete')
				.cancel()
				.addEvent('chainComplete',onComplete)
				.start({
				    'opacity': [1, 0], // Fade out
				    'left': [0, -1000]  // Slide out to left
				});
		}
		catch(e)
		{
			this.log.error(e);
			onComplete();
		}
	},
	
	fadeInBackground: function(code, onComplete)
	{
		try
		{
			var imgUrl = this.options.slides.get(code);
			
			$(this.options.backgroundId).setStyle('background-image', 'url('+imgUrl+')')
				.setStyle('visibility', 'visible').setStyle('opacity', 0);
			
			// Give this effect a small delay before starting
			(function(){
				
				this.backgroundFx
					.cancel()
					.start({
						'opacity': [0, 1]
					}).chain(onComplete);
				
			}.bind(this)).delay(150);
			
		}
		catch(e)
		{
			this.log.error(e);
			onComplete();
		}
	},
	
	load: function(request, container, mappedUrl, hijack, onComplete)
	{
		try{
			/**
			 * Set up a second BusyTracker to wait for the latter of the two events:
			 * - page load complete (success, failure, timeout, cancel)
			 * - ideal animation delay to expire (normally 2000ms, but set in options)
			 */
			var busyTracker = new BusyTracker(function(){
				this._loaded(container, hijack, onComplete);
			}.bind(this));

			// Add the delay task
			busyTracker.addTask(function(onComplete){
				onComplete.delay(this.options.idealDuration);
			}.bind(this));

			// Add the ajax load task
			busyTracker.addTask(function(onComplete){
				this._load(request, mappedUrl, onComplete);
			}.bind(this));

		} 
		catch (e)
		{
			this.log.error(e.message);
			onComplete();
		}
	},
	
	_load: function(request, mappedUrl, onCompleteCallback)
	{
		var url = mappedUrl.href;
		var dynamicContentOnly = 
			'?dynamicContentOnly='+(mappedUrl.dynamicContentOnly ? '1' : '0');
		
		request.setOptions({
			url: this.options.ajaxBaseUrl + url + dynamicContentOnly,
			
			onException: function(headerName, value){},
			
			onProgress: function(event, xhr){},
			
			onRequest: function()
			{
				Hijax.log.info('Base-page: Request started');
			},
			
			onComplete: function()
			{
				// Don't use onComplete here because it is fire before content is ready
				Hijax.log.info('Base-page: Request complete');
			},
			
			onTimeout: function()
			{
				Hijax.log.error('Base-page: Request timed out');
				var error = '<div id=Error404><strong>Oops</strong> page request timed out <em>'+url+'</em></div>';
				this.setResponse(error);
				onCompleteCallback();
			}.bind(this),
			
			onFailure: function(xhr)
			{
				var error = (xhr.status === 404)
					? '<div id=Error404><strong>404</strong> page not found <em>'+url+'</em></div>'
					: '<div id=Error500><strong>500</strong> error on page <em>'+url+'</em></div>';
				this.setResponse(error);
				onCompleteCallback();
			}.bind(this),
			
			onCancel: function()
			{
				Hijax.log.debug('Base-page: Request cancelled for ['+url+']');
				onCompleteCallback();
			},
			
			onSuccess: function(rTree, rElements, rHTML, rJavaScript)
			{
				Hijax.log.info('Base-page: Request success');
				this.setResponse(rHTML);
				onCompleteCallback();
			}.bind(this)
			
		});

		request.get();
		
		var gettingImpatient = function()
		{
			if (Hijax.busy)
			{
				window.fireEvent('navstillbusy');
			}
		};
		
		var expectedCompletionTime = 
			this.options.idealDuration + this.options.content.duration;
		
		gettingImpatient.delay(expectedCompletionTime);
		
	},
	
	setResponse: function(html)
	{
		this.log.debug('Response: '+html.substring(0, 15));
		this.responseHTML = html;
	},
	
	getResponse: function()
	{
		var response = this.responseHTML;
		this.responseHTML = null;
		return response;
	},
	
	_loaded: function(container, hijack, onComplete)
	{
		var responseHTML = this.getResponse();
		this.log.debug('Content: '+responseHTML.substring(0, 15));
		
		// Insert new content
		if (responseHTML)
		{
			this.options.contentSetter(container, responseHTML);
			hijack();
		}
		
		this.fadeOutBackground();
		
		// Fade page in
		this.content.removeEvents('chainComplete')
			.addEvent('chainComplete', onComplete)
			.start({
			    'opacity': [0, 1], // Fade in
			    'left': [1000, 0]  // Slide in from right
			});
	},
	
	fadeOutBackground: function()
	{
		// Fade out background
		this.backgroundFx
			.start({
				'opacity': [1, 0]
			});
	}
	
});
