if (!Website) throw "Requires: [host page.Website]";
/**
 * Website.DisplaySub
 * 
 * @package 	Website
 * @depends		Website
 */
Website.DisplaySub = new Class({
	
	Implements: [Options],
	
	options: {
		ajaxBaseUrl: null,
		containerFx: null,
		contentSetter: function(container, html)
		{
			container.empty().set('html', html);
		}
	},
	
	containerFx: null,
	log: $L.from('Website.DisplaySub'),
	
	initialize: function(options)
	{
		this.setOptions(options);
	},
	
	/**
	 * Organiser
	 * @param request
	 * @param container
	 * @param level
	 * @param url
	 * @param hijack
	 * @param code
	 * @param busyTracker
	 */
	swapPage: function(request, container, mappedUrl, hijack, code, busyTracker)
	{
		this.containerFx = new Fx.Tween(container, this.options.containerFx);
		
		/**
		 * BusyTracker waits for this one task:
		 */
		busyTracker.addTask(function(onComplete){
			this.load(request, container, mappedUrl, hijack, onComplete);
		}.bind(this));
	},
	
	load: function(request, container, mappedUrl, hijack, onComplete)
	{
		/**
		 * Set up a second BusyTracker to wait for the _load task
		 */
		var busyTracker = new BusyTracker(function(){
			this._loaded(container, hijack, onComplete);
		}.bind(this));
				
		// Add the ajax load task
		busyTracker.addTask(function(onComplete){
			this._load(request, mappedUrl, onComplete);
		}.bind(this));
	},
	
	_load: function(request, mappedUrl, onCompleteCallback)
	{
		var url = mappedUrl.href;
		var dynamicContentOnly = 
			'?dynamicContentOnly='+(mappedUrl.dynamicContentOnly ? '1' : '0');
		
		request.setOptions({
			url: this.options.ajaxBaseUrl + url + dynamicContentOnly,
			onException: function(headerName, value){},
			onProgress: function(event, xhr){},
			
			onRequest: function()
			{
				Hijax.log.info('Sub-page: Request started');
			},
			
			onComplete: function()
			{
				// Don't use onComplete here because it is fire before content is ready
				Hijax.log.info('Sub-page: Request complete');
			},
			
			onTimeout: function()
			{
				Hijax.log.error('Sub-page: Request timed out');
				var error = '<div id=Error404><strong>Oops</strong> page request timed out <em>'+url+'</em></div>';
				this.setResponse(error);
				onCompleteCallback();
			}.bind(this),
			
			onFailure: function(xhr)
			{
				var error = (xhr.status === 404)
					? '<div id=Error404><strong>404</strong> page not found <em>'+url+'</em></div>'
					: '<div id=Error500><strong>500</strong> error on page <em>'+url+'</em></div>';
				this.setResponse(error);
				onCompleteCallback();
			}.bind(this),
			
			onCancel: function()
			{
				Hijax.log.debug('Sub-page: Request cancelled for ['+url+']');
				onCompleteCallback();
			},
			
			onSuccess: function(rTree, rElements, rHTML, rJavaScript)
			{
				Hijax.log.info('Sub-page: Request success');
				this.setResponse(rHTML);
				onCompleteCallback();
			}.bind(this)
			
		});

		request.get();
	},
	
	setResponse: function(html)
	{
		this.log.debug('Response: '+html.substring(0, 15));
		this.responseHTML = html;
	},
	
	getResponse: function()
	{
		var response = this.responseHTML;
		this.responseHTML = null;
		return response;
	},
	
	_loaded: function(container, hijack, onComplete)
	{
		var responseHTML = this.getResponse();
		this.log.debug('Content: '+responseHTML.substring(0, 15));
		
		// Insert new content
		if (responseHTML)
		{
			var update = function(){
				this.options.contentSetter(container, responseHTML);
				hijack();
				this.containerFx.start(0, 1).chain(onComplete);
			}.bind(this);
			
			// Set up the fade in chain.
			this.containerFx.chain(update);
			
			// Start the fade out.
			this.containerFx.start(1, 0);
		}
	}
	
});
