if (!Website) throw "Requires: [host page.Website]";
/**
 * Website.Titler
 * 
 * Changes the website title depending on content loaded via AJAX. It works by
 * attempting to find a &lt;title&gt; element in the loaded content. It then strips
 * it before displaying the content.
 * 
 * Options:
 * title - the base title of the website
 * 
 * @package 	Website
 * @depends		Website
 */
Website.Titler = new Class({
	
	Implements: Options,
	
	options: {
		title: null,
		position: 'after',
		separator: ' - '
	},
	
	log: $L.from('Website.Titler'),
	
	initialize: function(options)
	{
		this.setOptions(options);
	},
	
	from: function(html)
	{
		try
		{
			// Base title
			var title = this.options.title;
			
			// Look for title in the html which actually has content (.+)
			var matches1 = html.match(/<title>(.+)<\/title>/g) || [];
			
			var deepestMatch = matches1.pop();
			
			if (deepestMatch)
			{
				var matches2 = deepestMatch.match(/<title>(.+)<\/title>/) || [];
				
				if (matches2 && matches2[1])
				{
					// Get the sub title
					var subtitle = matches2[1];
					
					// Prepare the title
					switch (this.options.position)
					{
					case 'before':
						title = subtitle+this.options.separator+title;
						break;
					default:
						title = title+this.options.separator+subtitle;
					}
					
					// Replace the <title> tag with a commented out version
					html = html.replace(deepestMatch, '<!-- Title: '+subtitle+' -->');
				}
				
				// Remove stray <title> tags including empty ones (.*)
				html = html.replace(/<title>(.*)<\/title>/, '');
			}
			
			// Replace html &amp; with plain text &
			title = title.replace(/&amp;/, '&');
			
			// Update the title
			document.title = title;
		} 
		catch (e)
		{
			this.log.error(e.message);
		}
		
		return html;
	}
	
});

