<?php defined('SYSPATH') or die('No direct script access.');

// -- Environment setup --------------------------------------------------------

// Load the core Kohana class
require SYSPATH.'classes/kohana/core'.EXT;

if (is_file(APPPATH.'classes/kohana'.EXT))
{
	// Application extends the core
	require APPPATH.'classes/kohana'.EXT;
}
else
{
	// Load empty core extensiond
	require SYSPATH.'classes/kohana'.EXT;
}

/**
 * Set the default time zone.
 *
 * @see  http://kohanaframework.org/guide/using.configuration
 * @see  http://php.net/timezones
 */
date_default_timezone_set('Africa/Nairobi');

/**
 * Set the default locale.
 *
 * @see  http://kohanaframework.org/guide/using.configuration
 * @see  http://php.net/setlocale
 */
setlocale(LC_ALL, 'en_US.utf-8');

/**
 * Enable the Kohana auto-loader.
 *
 * @see  http://kohanaframework.org/guide/using.autoloading
 * @see  http://php.net/spl_autoload_register
 */
spl_autoload_register(array('Kohana', 'auto_load'));

/**
 * Enable the Kohana auto-loader for unserialization.
 *
 * @see  http://php.net/spl_autoload_call
 * @see  http://php.net/manual/var.configuration.php#unserialize-callback-func
 */
ini_set('unserialize_callback_func', 'spl_autoload_call');

// -- Configuration and initialization -----------------------------------------

/**
 * Set the default language
 */
I18n::lang('en-us');

/**
 * Set Kohana::$environment if a 'KOHANA_ENV' environment variable has been supplied.
 *
 * Note: If you supply an invalid environment name, a PHP warning will be thrown
 * saying "Couldn't find constant Kohana::<INVALID_ENV_NAME>"
 */
if (isset($_SERVER['REDIRECT_KOHANA_ENV']))
{
	Kohana::$environment = constant('Kohana::'.strtoupper($_SERVER['REDIRECT_KOHANA_ENV']));
}

/**
 * Initialize Kohana, setting the default options.
 *
 * The following options are available:
 *
 * - string   base_url    path, and optionally domain, of your application   NULL
 * - string   index_file  name of your index file, usually "index.php"       index.php
 * - string   charset     internal character set used for input and output   utf-8
 * - string   cache_dir   set the internal cache directory                   APPPATH/cache
 * - boolean  errors      enable or disable error handling                   TRUE
 * - boolean  profile     enable or disable internal profiling               TRUE
 * - boolean  caching     enable or disable internal caching                 FALSE
 */

$base_url = '';
switch(Kohana::$environment)
{
	case Kohana::PRODUCTION:
		$base_url = 'http://diveinkenya.com/';
		break;
	case Kohana::STAGING:
		$base_url = 'http://staging.diveinkenya.com/';
		break;
	default:
		$base_url = 'http://diveinkenya.vm/';
}

Kohana::init(array(
	'base_url'   => $base_url,
	'index_file' => FALSE,
	'errors'	 => TRUE,
	'profile'    => Kohana::$environment !== Kohana::PRODUCTION,
    'caching'    => Kohana::$environment === Kohana::PRODUCTION,
));

unset($base_url);

if(substr( $_SERVER['REQUEST_URI'], 0, strlen( '/min/g=' ) ) == '/min/g=')
{
	session_cache_expire(60 * 24 * 365);
	session_cache_limiter('public');
}

Cookie::$salt = '15DE3347AF5232843125EA1E1F3B8ADB4B404473E4939F3BC1FD869C23575BCD';
Session::$default = 'database';

/**
 * Attach the file write to logging. Multiple writers are supported.
 */
Kohana::$log->attach(new Log_File(APPPATH.'logs'));

/**
 * Attach a file reader to config. Multiple readers are supported.
 */
Kohana::$config->attach(new Config_File);




/**
 * Enable modules. Modules are referenced by a relative or absolute path.
 */
Kohana::modules(array(
	// Platform modules
	'mss-platform' 	 				=> MODPATH.'mss-platform',				// MSS Platform
	'mss-platform-core' 			=> MODPATH.'mss-platform-core',			// MSS Platform Core Module
	'mss-platform-user'				=> MODPATH.'mss-platform-user',			// MSS Platform User Module (after mss-platform-backend)
	'mss-platform-hijax'			=> MODPATH.'mss-platform-hijax',		// MSS Platform Hijax Module

	// External modules
	'mss-assets' 	 				=> MODPATH.'mss-assets',				// MSS Assets Module
	'mss-hijax' 	 				=> MODPATH.'mss-hijax',					// MSS Hijax Module
	'mss-message' 	 				=> MODPATH.'mss-message',				// MSS Message Module
	'mss-minify' 	 				=> MODPATH.'mss-minify',				// MSS Minify Module
	'mss-history'					=> MODPATH.'mss-history',				// MSS History Manager Module
	'mss-jslogger'					=> MODPATH.'mss-jslogger',				// MSS JSLogger Module
	'mss-google-analytics'			=> MODPATH.'mss-google-analytics',		// MSS Google Analytics Module
	'mss-website'					=> MODPATH.'mss-website',				// MSS Website Module
	'mss-yahoo-currency'			=> MODPATH.'mss-yahoo-currency',		// MSS Yahoo Currency Module

	'kohana-kostache'				=> MODPATH.'kohana-kostache',			// Kohana Kostache
	'assets' 	 					=> MODPATH.'assets',				// Assets Module
	'arden'							=> MODPATH.'arden',
	'kohana-user'					=> MODPATH.'user/login',

	// Core modules
	'auth'       					=> MODPATH.'auth',       				// Basic authentication
	// 'cache'      				=> MODPATH.'cache',      				// Caching with multiple backends
	// 'codebench'  				=> MODPATH.'codebench',  				// Benchmarking tool
	'database'   					=> MODPATH.'database',   				// Database access
	// 'image'      				=> MODPATH.'image',      				// Image manipulation
	'orm'        					=> MODPATH.'orm',        				// Object Relationship Mapping
// 	'unittest'   					=> MODPATH.'unittest',   				// Unit testing
	'userguide' 	 				=> MODPATH.'userguide',  				// User guide and API documentation
	'swift'							=> MODPATH.'swift',					// Kohana Kostache
));

Route::set('sitemap', 'sitemap.xml')
	->defaults(array(
		'controller' => 'sitemap',
		'action'     => 'index',
	));

Route::set('static_page', '<controller>(/<action>)',
	array(
		'controller' => 'voucher|test', // for static controllers, use this route
	))
	->defaults(array(
		'action'     => 'index',
	));

Route::set('downloads', 'downloads/<action>')
	->defaults(array(
		'directory' => 'pages',
		'controller' => 'downloads',
	));

Route::set('error', 'error/<action>/<origuri>/<message>', array('action' => '[0-9]++', 'origuri' => '.+', 'message' => '.+'))
->defaults(array(
'controller' => 'error',
'action'     => 'index'
));

Route::set('admin_pages', 'administrator/<controller>(/<action>(/<id>))')
	->defaults(array(
		'directory'  => 'admin',
		'action'     => 'index',
	));

	Route::set('administrator', 'administrator(/<action>(/<id>))')
	->defaults(array(
	'directory'  => 'admin',
	'action'     => 'index',
	'controller' => 'administrator',
	));

Route::set('voucher', 'rates/vouchers/<action>(/<id>)',
	array(
		'action' => 'checkout|return|cancel',
	))
	->defaults(array(
		'directory'		=> 'pages/rates',
		'controller'	=> 'vouchers',
		'action'		=> 'index',
	));

Route::set('hijax/content', 'pages/<controller>(/<action>(/<id>))')
	->defaults(array(
		'directory'		=> 'pages',
		'controller'	=> 'default',
		'action'		=> 'index',
	));


/**
 * Set the route for hijax.
 * Number of levels is completely arbitrary.
*/
Route::set('hijax/base', '(<level0>(/<level1>(/<level2>(/<level3>(/<level4>)))))')
	->defaults(array(
		'controller' 	=> 'hijax',
		'action'     	=> 'index',
	));

Route::set('updateprices', 'updateprices(/<action>)')
	->defaults(array(
		'controller' => 'updateprices',
		'action'     => 'index',
	));

Route::set('pdf', 'reports(/<action>)')
	->defaults(array(
		'controller' => 'reports',
		'action'     => 'prices',
	));

Route::set('lightbox', 'lightbox(/<id>)')
	->defaults(array(
		'controller' => 'lightbox',
		'action'     => 'index',
	));

Route::set('checkout', 'checkout(/<action>)')
	->defaults(array(
		'controller' => 'checkout',
		'action'     => 'index',
	));

Route::set('level2', '<directory>(/<controller>(/<action>(/<id>)))',
	array('directory' => ('admin')))
	->defaults(array(
		'directory'  => 'admin',
		'controller' => 'default',
		'action'     => 'index',
	));

// Route::set('level1', '<controller>(/<action>(/<id>))')
// ->defaults(array(
// 		'controller' => 'welcome',
// 		'action'     => 'index',
// ));

/**
 * Set the routes. Each route must have a minimum of a name, a URI and a set of
 * defaults for the URI.
 */
Route::set('default', '(<controller>(/<action>(/<id>)))')
	->defaults(array(
		'controller' => 'welcome',
		'action'     => 'index',
	));

require APPPATH.'vendor'.DIRECTORY_SEPARATOR.'paypal'.DIRECTORY_SEPARATOR.'paypalfunctions.php';
