<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Pages_Blog extends Controller_Hijax implements HasMenu {

	public $template = 'pages/blog';
	public $subtitle = 'Adventure Blog';
	
	public function action_index()
	{
		$this->template->menu = Menu::factory($this);
		$this->render_dynamic($this->blog($this->request->param('id')));
	}
	
	private function blog($id)
	{
		if (isset($id))
		{
			return $this->blogpost($id);
		}
		else
		{
			return $this->blogposts();
		}
	}
	
	private function blogpost($id)
	{
		$node = Node::factory('blogpost', NULL, array('author', 'date', 'code', 'summary'))
			->where_option('code', '=', $id)
			->published()
			->find();
		
		if ( ! $node->loaded())
		{
			throw new HTTP_Exception_404('The requested resource :uri was not found on this server.',
			array(':uri' => $this->request->uri()));
		}
		
		$view = new View_Pages_Blog_Post();
		
		$date = array(
			'pretty' => date('j M Y', $node->option('date')),
			'ugly' => date('Y-m-d', $node->option('date')),
		);
		
		$fb_comments = Iframe::insert('iframes/fb-blogcomments', 'width=695 scrolling=no autoresize', array(
			'app_id' => '110150742396352',
			'href' => Website::instance()->current_url(),
		));
		
		$text = $this->get_blog_text($node->blogpost->text);
				

		$view->set('node', $node)
			 ->set('text', $text)
			->set('date', $date)
// 			->set('profiler', View::factory('profiler/stats'))
			->set('fb_comments', $fb_comments);
		
		// Return array
		return array($node->name, $view);
	}
	
	protected function get_blog_text($text)
	{
		$parser_path = APPPATH.'vendor/markdown/markdown.php';
		include_once $parser_path;
		
		$html=Markdown($text); 
		
		//return htmlentities($html); //should be done this way but displays text instead of html
		return $html;
	}
	
	private function blogposts()
	{
		$latest = Node::factory('blogpost', NULL, array('code'))
			->published()
			->order_by_option('date', 'DESC')
			->find();

		return $this->blogpost($latest->option('code'));
	}
	
	public function menu_data($identifier)
	{
		// Load the blogposts to make the menu
		return Node::factory('blogpost', NULL, array('author', 'code'))
			->published()
			->order_by('created', 'DESC')
//			->order_by_option('date', 'DESC')
			->find_all();
// 			->domain_find_all();
	}
	
	public function menu_item($item, $identifier)
	{
		$href = '/blog/'.$item->option('code');
		$date = $item->created;
		$display_date = isset($date) ? Date::fuzzy_span($date) : '';
		$text = "$item->name <p>$display_date</p>";
		
		return array(
			'id' 	=> null,
			'class' => 'Blog-menu-item',
			'href'	=> $href,
			'text' 	=> $text,
		);
	}
	
	public function is_active($item, $current, $identifier)
	{
		return $current === $item['href'];
	}
	
}
