<?php defined('SYSPATH') or die('No direct script access.');
/**
 * HTML helper class. Provides generic methods for generating various HTML
 * tags and making output HTML safe.
 *
 * @package    Kohana
 * @category   Helpers
 * @author     Kohana Team
 * @copyright  (c) 2007-2011 Kohana Team
 * @license    http://kohanaframework.org/license
 */
class HTML extends Kohana_HTML {

	public static function script($file, array $attributes = NULL, $protocol = NULL, $index = FALSE)
	{
		// Set the script link
		$attributes['src'] = strpos($file, '://') === FALSE ? '/'.$file : $file;

		// Set the script type
		$attributes['type'] = 'text/javascript';
	
		return '<script'.HTML::attributes($attributes).'></script>';
	}	
	
	public static function style($file, array $attributes = NULL, $protocol = NULL, $index = FALSE)
	{
		// Set the stylesheet link
		$attributes['href'] = strpos($file, '://') === FALSE ? '/'.$file : $file;
	
		// Set the stylesheet rel
		$attributes['rel'] = 'stylesheet';
	
		// Set the stylesheet type
		$attributes['type'] = 'text/css';
	
		return '<link'.HTML::attributes($attributes).' />';
	}
	
	public static function anchor($uri, $title = NULL, array $attributes = NULL, $protocol = NULL, $index = TRUE)
	{
		if ($title === NULL)
		{
			// Use the URI as the title
			$title = $uri;
		}
	
		if ($uri === '')
		{
			// Only use the base URL
			$uri = URL::base($protocol, $index);
		}
		else
		{
			if (strpos($uri, '://') !== FALSE)
			{
				if (HTML::$windowed_urls === TRUE AND empty($attributes['target']))
				{
					// Make the link open in a new window
					$attributes['target'] = '_blank';
				}
			}
			elseif ($uri[0] !== '#')
			{
				$path = preg_replace('~^[-a-z0-9+.]++://[^/]++/?~', '', trim($uri, '/'));
				
				if ( ! UTF8::is_ascii($path))
				{
					// Encode all non-ASCII characters, as per RFC 1738
					$path = preg_replace('~([^/]+)~e', 'rawurlencode("$1")', $path);
				}
				// Make the URI absolute for non-id anchors
				$uri = $path;
				if(!preg_match('/^\//', $uri) && !preg_match('/^http/', $uri))
				{
					$uri = '/'.$uri;
				}
			}
		}
	
		// Add the sanitized link to the attributes
		$attributes['href'] = $uri;
	
		return '<a'.HTML::attributes($attributes).'>'.$title.'</a>';
	}
}