if (!Hijax.Abstract) throw "Requires: [Hijax.Abstract]";
/**
 * Hijax.Old
 * 
 * Without HTML5's History API implementation. Uses mss-history module.
 * 
 * @package 	Hijax
 * @depends		Hijax
 */
Hijax.Old = new Class({
	
	Extends: Hijax.Abstract,
	
	initHistory: function()
	{
		Hijax.log.info('Initialising Hijax History in Old mode.');
		
		// Handle history change via the History manager.
		History.instance().addListener(function(state){
			
			// Correct for empty state.
			var url = state || '/';
			// If state actually changed then call onStateChanged
			this.ifStateChanged(url, function(){
				// State was changed and requires an ajax call.
				this.onStateChanged(url);
			}.bind(this));
		}.bind(this));
	},
	
	ifStateChanged: function(url, callback)
	{
		// Check if this is your first state change, potentially invoking ajax
		if (this.initial)
		{
			// It is your first state change, test what page you're looking for
			if (url !== '/')
			{
				/*
				 * It the first state change and you've requested a page that is
				 * not the home page, so replace the initial content with a loading
				 * message and then proceed with notifying that the state changed.
				 * 
				 * was: container.set('html', this.options.loading);
				 */
				this.ifContainer(0, function(container){
					container.empty();
				});
				
				callback();
			}
			else
			{
				/*
				 * We still need to fire the historychange event.
				 */
				this.fireHistoryChange(url);
			}
			
			// Turn off the flag
			this.initial = false;
		}
		else
		{
			// Behave like normal if this is not the initial state change
			callback();
		}
	},
	
	handleAnchorClick: function(href)
	{
		window.location.hash = href;
	},
	
	updateReferrer: function()
	{
		this.referrer = window.location.hash;
	},
	
	/**
	 * Redirect, if necessary, before initialising the Hijax system
	 * 
	 * If in Old mode, hijax wants to be on the base page from where it uses #'s.
	 *  
	 * If in HTML5 mode and someone has referred you to a link that looks like /#...
	 * then redirect to the new version.
	 */
	redirect: function()
	{

		var path = window.location.pathname;
		var cookieKey = 'hijax.redirect';

		// Read and consume the URL state cookie
		var redirect = ""+(Cookie.read(cookieKey) || '/');
		Cookie.dispose(cookieKey);

		// You're not on /, so redirect back to / but store state in cookie
		if('/' !== path)
		{
			// Write URL state to a cookie
			Cookie.write(cookieKey, path);

			// Now do the actual redirect
			window.location.href = '/';

			/*
			 * A hard redirect just occurred, so return true to notify the caller 
			 * not to continue with any more javascript execution on this page.
			 * (This is because javascript doesn't have an exit statement like php)
			 */
			return true;
		}

		// The cookie shows that you need to be "redirected" to a hash #/prev/page
		if('/' !== redirect)
		{
			window.location.hash = redirect;
		}

		// If the URL is / or /# then complete it by making it /#/
		if (window.location.hash.replace(/^#/, '') === '')
		{
			window.location.hash = '/';
		}

		/*
		 * No hard redirect occurred, so we're going to stay on this page and allow
		 * the history layer to take over navigation from here.
		 */
		return false;
	}
	
});