<?php defined('SYSPATH') or die('No direct script access.');

class Controller_Hijax extends Hijax {

	public $template = 'template/hijax';

		
	public function before()
	{
		parent::before();
		$this->template->subtitle = isset($this->subtitle) ? $this->subtitle : '';
	}
	
	public function action_index()
	{
		$level0 = $this->request->param('level0');
		$level1 = $this->request->param('level1');
		$level2 = $this->request->param('level2');
		
		$route = Route::get('hijax/content');
		
		$action = $this->get_action($route);
		$default_action = Arr::get($route->defaults(), 'action');
		
		$uri = $route->uri(array(
			'controller' 	=> $level0,
			'action' 		=> isset($action) ? $action : $default_action,
			'id'			=> isset($action) ? $level2 : $level1,
		));
		
		$content = Request::factory($uri)->execute();
		
		$subtitle = $this->extract_subtitle($content);
		
		$this->response($content, $subtitle);
	}
	
	protected function get_action(Route $route) 
	{
		$level0 = $this->request->param('level0');
		$level1 = $this->request->param('level1');
		
		if (isset($level1))
		{
			$action = $level1;
			$method = 'action_'.$action;
			
			$fullpath = $this->get_fullpath($level0);
			
			$class = $this->get_class($fullpath);
			
			if (isset($class) && $class->hasMethod($method))
			{
				return $action;
			}
		}
		
		return NULL;
	}
	
	protected function subrequest($path, $level)
	{
		if ($this->is_ajax())
		{
			$subrequest = $this->get_subrequest();
		}
		else
		{
			$subrequest = Request::initial()->param('level'.$level);
		}

		if (!empty($subrequest))
		{
			$subrequest_url = $path.'_'.$subrequest;
		}
		else
		{
			$subrequest_url = $path.'/default';
// 			$subrequest_url = $path.'_default';
		}

		return Request::factory( $subrequest_url )->execute();
	}

	protected function is_ajax()
	{
		$is_ajax = Request::initial()->is_ajax();

		if (Kohana::$environment > Kohana::PRODUCTION)
		{
			$is_ajax = $is_ajax || Request::initial()->query('ajax');
		}
		return $is_ajax;
	}
	
	protected function get_subrequest()
	{
		$param = 'id';
		$glue = '_';
			
		// Get the subpath param from the initial request
		$subpath = Request::initial()->param($param);
			
		// Explode it to get the parts
		$subpath_parts = explode($glue, $subpath);
			
		// Consume the first element in the array
		$subrequest = array_shift($subpath_parts);
			
		// Implode the remaining array elements into a string
		$subpath = implode($glue, $subpath_parts);
			
		// Tamper with the request to set back the deminished subpath
		Request::initial()->set_param($param, $subpath);
		
		return $subrequest;
	}
	
	protected function get_class($fullpath)
	{
		try
		{
			return new ReflectionClass($fullpath);
		}
		catch (ReflectionException $e) 
		{
			return null;
		}
	}
	
	protected function get_fullpath($uri)
	{
		$path = $uri === '/' ? 'default' : $uri;
		
		$prefix = 'controller_';
		$directory = 'pages_';
		
		// Concatenate and convert to controller_pages_rates_padi_default
		$fullpath = str_replace('/', '_', $prefix.$directory.$path);
		
		// If the class has a _default in its subfolder then use that subtitle
		$fullpath .= class_exists($fullpath.'_default') ? '_default' : '';
		
		return $fullpath;
	}
	
	protected function extract_subtitle($content)
	{
		if (preg_match_all('#<title>(.+)</title>#U', $content, $matches))
		{
			return array_pop($matches[1]);
		}
	}

} // End Hijax
