<?php defined('SYSPATH') or die('No direct script access.');
/**
 * Extends Kohana_Route to provide another level of Route matching necessary for
 * the Hijax module.
 *
 * @package    MSS
 * @category   Hijax
 * @author     Paul Schwarz
 * @copyright  &copy; 2011 www.mss.co.ke
 */
class Hijax_Route extends Kohana_Route {

	/**
	* Tests if the route matches a given URI. A successful match will return
	* all of the routed parameters as an array. A failed match will return
	* boolean FALSE.
	*
	*     // Params: controller = users, action = edit, id = 10
	*     $params = $route->matches('users/edit/10');
	*
	* This method should almost always be used within an if/else block:
	*
	*     if ($params = $route->matches($uri))
	*     {
	*         // Parse the parameters
	*     }
	*
	* @param   string  URI to match
	* @return  array   on success
	* @return  FALSE   on failure
	*/
	public function matches($uri)
	{
		// Call the parent's matches function.
		$params = parent::matches($uri);
		
		if ($params['controller'] !== 'hijax')
		{
			// If the URI doesn't match hijax you've found a correct route.
			return $params;
		}
		elseif ($this->is_hijax($uri, $params))
		{
			// If the URI matches hijax you need to test if it really is hijax.
			return $params;
		}
		else 
		{
			// If it's not a valid hijax controller then keep looking.
			return FALSE;
		}
	}
	
	/**
	 * Test if a URI identifies a hijax page.
	 * 
	 * The hijax "base" route obscured Kohana's "default" route so it catches more
	 * URI matches than it should. Therefore you need to test if a matched route
	 * really is a valid hijax route, otherwise keep looking.
	 * 
	 * The hijax "base" route's first element is called level0. Hijax route matches
	 * see controller names represented by level0. Hijax pages live in <code>
	 * classes/controller/pages</code>, so simply checking for the existence of a
	 * class called <code>controller_pages_&lt;level0&gt;</code> should be enough
	 * to know if the route should be considered as hijax or ignored.
	 * 
	 * @param string $uri
	 * @param array $params
	 */
	protected function is_hijax($uri, $params)
	{
		// Hijax specific magic variables
		$level0 = 'level0';
		$directory = 'pages';
		
		// If you are on the base URL / then level0 is not set but it is valid hijax
		if (!isset($params[$level0]))
		{
			return $params;
		}
		
		// Create the class prefix (from Kohana)
		$prefix = 'controller_';
		
		// Controller
		$controller = $params[$level0];
		
		if ($directory)
		{
			// Add the directory name to the class prefix
			$prefix .= str_replace(array('\\', '/'), '_', trim($directory, '/')).'_';
		}
		
		return class_exists($prefix.$controller);
	} 


} // End Route
