<?php defined('SYSPATH') or die('No direct script access.');
/**
 * Minify wrapper. This is an almost complete wrapper around Mr Clay's Minify project.
 * 
 * Usage:
 * <ul>
 *   <li>/min/g=js&version=1 (preferred)</li>
 *   <li>/min/?g=js&version=1</li>
 *   <li>/min/?f=scripts/jquery-1.2.6.js,scripts/site.js,scripts/home.js</li>
 * </ul>
 *   ... and so on.
 *
 * @package    	MSS
 * @category	Minify
 * @author     	Paul Schwarz
 * @version 	1.0
 * @copyright  	&copy; 2011 Media &amp; Software Studio
 */

class Kohana_Controller_Minify extends Controller {

	public function action_index()
	{
		$query_string = $this->request->param('query_string');
		
		// Prepare the $_GET array by forcing the query string into the array
		$this->_prepare($query_string);

		// Hand over to the minify library in quiet mode
		$min = $this->_minify();
		
		if (isset($min['headers']['Content-Length'])){
			$min['headers']['Content-Length'] = (string) $min['headers']['Content-Length'];
		}
		
		foreach($min['headers'] as $key => $val)
		{
			$this->response->headers($key, $val);
		}
		
		//$minify_config = Kohana::$config->load('minify');
		//$this->response->headers('Cache-Control', 'max-age='.$minify_config->get('min_maxAgeProduction').', public');
		$this->response
			->status($min['statusCode'])
			->body($min['content']);
	}
	
	private function _prepare($query_string)
	{
		// $_GET is empty if request looks like /min/g=css instead of /min/?g=css
		if (empty($_GET))
		{
			$query_string_clean = str_replace('&amp;', '&', $query_string);
			$query_parts = explode('&', $query_string_clean);
			foreach ($query_parts as $part)
			{
				$param = explode('=', $part);
				
				// If a key and value pair are found and the key is a valid value then add it to $_GET
				if (isset($param[0]) && isset($param[1]) && in_array($param[0], array('debug', 'g', 'f', 'd')))
				{
					$_GET[$param[0]] = $param[1];
				}
			}
		}
	}
	
	private function _minify()
	{
		// Instead of front controller for default Minify implementation
		
		$minify_config = Kohana::$config->load('minify');
		
		$min_documentRoot 			= DOCROOT;
		$min_uploaderHoursBehind 	= $minify_config->get('min_uploaderHoursBehind');
		$min_cachePath 				= $minify_config->get('min_cachePath');
		$min_cacheFileLocking 		= $minify_config->get('min_cacheFileLocking');
		$min_symlinks 				= $minify_config->get('min_symlinks');
		$min_allowDebugFlag 		= $minify_config->get('min_allowDebugFlag');
		$min_errorLogger 			= $minify_config->get('min_errorLogger');
		$min_groups 				= $minify_config->get('groups');
		
		$min_serveOptions['bubbleCssImports'] = $minify_config->get('min_bubbleCssImports');
		
		$min_serveOptions['maxAge'] = Kohana::$environment <= Kohana::STAGING 
			? $minify_config->get('min_maxAgeProduction') 
			: $minify_config->get('min_maxAgeTesting');
		
		//$min_serveOptions['minApp']['allowDirs'] = array('//js', '//css');
		
		$min_serveOptions['minApp']['groupsOnly'] = $minify_config->get('min_groupsOnly');
		
		$min_serveOptions['minApp']['maxFiles'] = $minify_config->get('min_maxFiles');
		
		ini_set('zlib.output_compression', '0');
		
		if (!is_dir($min_cachePath))
		{
			mkdir($min_cachePath);
		}
		
		Minify::$uploaderHoursBehind = $min_uploaderHoursBehind;
		
		Minify::setCache(
		    isset($min_cachePath) ? $min_cachePath : ''
		    ,$min_cacheFileLocking
		);
		
		if ($min_documentRoot) {
		    $_SERVER['DOCUMENT_ROOT'] = $min_documentRoot;
		} elseif (0 === stripos(PHP_OS, 'win')) {
		    Minify::setDocRoot(); // IIS may need help
		}
		
		$min_serveOptions['minifierOptions']['text/css']['symlinks'] = $min_symlinks;
		
		if ($min_allowDebugFlag && isset($_GET['debug'])) {
		    $min_serveOptions['debug'] = true;
		}
		
		if ($min_errorLogger) {
		    require_once 'Minify/Logger.php';
		    if (true === $min_errorLogger) {
		        require_once 'FirePHP.php';
		        Minify_Logger::setLogger(FirePHP::getInstance(true));
		    } else {
		        Minify_Logger::setLogger($min_errorLogger);
		    }
		}
		
		// check for URI versioning
		if (preg_match('/&\\d/', $_SERVER['QUERY_STRING'])) {
		    $min_serveOptions['maxAge'] = 31536000;
		}
		if (isset($_GET['g'])) {
		    // well need groups config
			$min_serveOptions['minApp']['groups'] = $min_groups;
		}
		if (isset($_GET['f']) || isset($_GET['g'])) {
		    // serve in quiet mode so Kohana can take the result and respond accordingly
			$min_serveOptions['quiet'] = true;
			return Minify::serve('MinApp', $min_serveOptions);
		} else {
		    header("Location: /");
		    exit();
		}
				
	}

} // End Minify
