<?php defined('SYSPATH') or die('No direct script access.');

class Session_Database extends Kohana_Session_Database {
	
	/**
	 * Override _write function to store the Auth::instance()->get_user()->id
	 * @see Kohana_Session_Database::_write()
	 */
	protected function _write()
	{
		if ($this->_update_id === NULL)
		{
			// Insert a new row
			$query = DB::insert($this->_table, $this->_columns)
				->values(array(':new_id', ':active', ':contents', ':user_id'));
		}
		else
		{
			// Update the row
			$query = DB::update($this->_table)
				->value($this->_columns['last_active'], ':active')
				->value($this->_columns['contents'], ':contents')
				->value($this->_columns['user_id'], ':user_id')
				->where($this->_columns['session_id'], '=', ':old_id');
	
			if ($this->_update_id !== $this->_session_id)
			{
				// Also update the session id
				$query->value($this->_columns['session_id'], ':new_id');
			}
		}
		
		$user = Auth::instance()->get_user();
		$auth_user_id = $user ? $user->id : NULL;
		
		$auth_user_id = (is_numeric($auth_user_id) && intval($auth_user_id) > 0) ? 
			$auth_user_id : NULL;
		
		$query
			->param(':new_id',   $this->_session_id)
			->param(':old_id',   $this->_update_id)
			->param(':active',   $this->_data['last_active'])
			->param(':contents', $this->__toString())
			->param(':user_id',  $auth_user_id);
		
		// Execute the query
		$query->execute($this->_db);
	
		// The update and the session id are now the same
		$this->_update_id = $this->_session_id;
	
		// Update the cookie with the new session id
		Cookie::set($this->_name, $this->_session_id, $this->_lifetime);
	
		return TRUE;
	}
	
} // End Session_Database
