<?php defined('SYSPATH') or die('No direct script access.');

abstract class Kohana_Controller_App_Template_CRUD extends Controller_App_Template {
	
	/**
	 * The name of the model having a database view for loading rows.
	 * @var string
	 */
	protected $model_name = '';
	
	public function action_data()
	{
		$this->auto_render = FALSE;
		echo $this->data();
	}
	
	public function action_action()
	{
		$this->auto_render = FALSE;
	
		$payload = $this->request->post('payload');
	
		$data = json_decode(base64_decode($payload));
	
		$action = $data->action;
		$ids = $data->ids;
		$args = $data->args;
	
		echo $this->{'ajax_'.$action}($ids, $args);
	}
	
	public function action_edit()
	{
		$req = $this->request;
		$id = $req->param('id');
		$toolbar = $this->toolbar('edit');
		
		if(empty($this->model_name))
		{
			throw new Kohana_Exception('For CRUD operations you must set the model_name for the controller');
		}
		
		$errors = NULL;
	
		if (! empty($_POST))
		{
			if (is_numeric($id))
			{
				// Edit: load existing node.
				$node = Node::factory($this->model_name, $id)
					->domain_find();
			}
			else
			{
				// Create: make new node.
				$node = Node::factory($this->model_name);
			}
				
			// Set values
			$this->edit_values($node, $req);
			
			// Save
			try
			{
				$result = $node->save();
				
				// message: save success
				Message::add('success', __('Saved.'));
				
				// Then save/close or save/apply
				$this->edit_done($req, $node->id);
			}
			catch (ORM_Validation_Exception $e)
			{
				$debug = ! TRUE ? Debug::vars($e->errors($this->model_name)) : '';
				
				Message::add('error', __('Please correct errors.'.$debug));
				
				$errors = $e->errors($this->model_name);
				
				$editor = $this->edit_editor($node, $errors);
			}
		}
		
		if ( ! isset($editor))
		{
			// If an ID is set, load the information
			if (is_numeric($id))
			{
				$node = $this->edit_load($id);
					
				if (! $node->loaded())
				{
					$toolbar = new View_Data_DataToolbar();
					$editor = 'Not found';
					$this->response->status(404);
				}
				else
				{
					// Editor view
					$editor = $this->edit_editor($node, $errors);
				}
			}
			// New
			else 
			{
				$editor = $this->edit_editor(NULL, $errors);
			}
		}
		
		Template::instance($this)->render(array(
			'subtitle'	=> $this->edit_subtitle( ! is_numeric($id)),
			'toolbar'	=> $toolbar,
			'article'	=> $editor,
		));
	}
	
	protected function edit_done($req, $id)
	{
		if ('close' === $req->post('then'))
		{
			// Then close
			$redirect = $this->route();
		}
		else
		{
			// Then refresh
			$return = ! $req->query('return') ? '' : URL::query(array('return' => $req->query('return')));
			$redirect = $this->route(FALSE, 'edit', $id).$return;
		}
		$req->redirect($redirect);
	}
	
	protected function edit_values(&$node, Request $request)
	{
		$node->values($request->post());
	}
	
	protected function edit_load($id)
	{
		return Node::factory($this->model_name, $id)->domain_find();
	}
	
	protected function edit_editor($node, $errors){}
	
	protected function edit_subtitle($new){}
	
	/**
	 * Load data from the database view. Override this to load data in a customised
	 * fashion or to handle non-standard query criteria.
	 */
	protected function data()
	{
		return ORM::factory($this->model_name)
			->datatable($this->request->query('config'), $this->fields());
	}
	
	protected function ajax_action_delete($ids)
	{
		Node::factory($this->model_name)->domain_delete($ids);
	}
	
}