<?php defined( 'SYSPATH' ) or die( 'No direct script access.' );

class Kohana_Website {
	
	protected static $instance;
	
	public static function instance()
	{
		if (Website::$instance === NULL)
		{
			Website::$instance = new Website;
		}
		return Website::$instance;
	}
	
	protected $config;
	
	protected function __construct()
	{
		$config_file = 'website';
		$this->config = Kohana::$config->load($config_file);
	}
	
	public final function head($styles=array(), $scripts=array())
	{
		return $this->render_head($styles, $scripts);
	}
	
	public function title($subtitle=NULL)
	{
		$title = $this->config->get('title');
		$separator = $this->config->get('title_separator');
		
		// Prepare the title
		if (isset($subtitle) && !empty($subtitle))
		{
			switch ($this->config->get('title_position'))
			{
				case 'before':
					$title = $subtitle.$separator.$title;
					break;
				default:
					$title = $title.$separator.$subtitle;
			}
		}
		
		return $title;
	}
	
	public final function mainmenu()
	{
		$items = $this->config->get('mainmenu');
		
    	foreach($items as $key => $val)
    	{
    		if ($val['type'] === 'item')
    		{
    			$menu_items[] = $this->mainmenu_item($key, $val);
    		}
    	}
    	
    	$menu = implode(PHP_EOL, $menu_items);
		return $menu;
	}
	
	/**
	 * Override this function to provide different markup for your menu item
	 * @param string $menu_item_key
	 * @param array $menu_item
	 */
	protected function mainmenu_item($menu_item_key, $menu_item)
	{
		$html = $menu_item['name'].' <em style=\'left:'.$menu_item['left'].'px;\'>'.$menu_item['desc'].'</em>';
		return HTML::anchor('/'.$menu_item['href'], $html);
	}
	
	public function menu_detail($key, $detail)
	{
		return $this->config['mainmenu'][$key][$detail];
	}
	
	public function current_url($path=NULL)
	{
		$url = 'http://'.$_SERVER['HTTP_HOST'];
		if (isset($path))
		{
			$url .= $path;
		}
		else
		{
			$url .= preg_replace('#^/pages#', '', $_SERVER['REDIRECT_URL']);
		}
		return $url;
	}
	
	protected function render_head($styles, $scripts)
	{
		$head = '';
		
		$head .= $this->css($styles);
    	
		if (!empty($this->config['fonts'])){
    		$head .= "\t\t".HTML::style($this->config->get('fonts')).PHP_EOL;
    	}
    	
    	if (!empty($this->config['js_framework'])){
    		$url = $this->config->get('js_framework');
    		if (preg_match('#^//#', $url)){
    			$url = preg_replace('#^//#', '/', $url);
    		}
    		$head .= "\t\t".HTML::script($url).PHP_EOL;
    		
    		$fallback = $this->config['js_framework_fallback'];
    		if (!empty($fallback)){
    			$head .= "\t\t"."<script>window.MooTools || document.write('<script src=\"$fallback\"><\/script>')</script>".PHP_EOL; 			
    		}
    	}
		
		$head .= $this->above_javascript();
		
		if (!empty($this->config['js'])){
	    	$head .= "\t\t".HTML::script($this->config->get('js')).PHP_EOL;
	    }
	    
	    foreach ($scripts as $file)
	    {
	    	$head .= "\t\t".HTML::script($file).PHP_EOL;
	    }
		
		return $head;
	}
	
	public function css(array $styles = array())
	{
		$head = '';
		
		if (!empty($this->config['css'])){
			$head .= HTML::style($this->config->get('css')).PHP_EOL;
		}
		
		foreach ($styles as $file => $type)
		{
			$head .= "\t\t".HTML::style($file, array('media' => $type)).PHP_EOL; 
		}
		
		return $head;
	}
	
	protected function above_javascript()
	{
		$modules = Kohana::modules();
		
		$result = '';
		
		if (array_key_exists('mss-jslogger', $modules)){
			$result .= View::factory('jslogger');
		}
		
		if (array_key_exists('mss-google-analytics', $modules)){
			$result .= View::factory('analytics');
		}

		$result .= $this->javascript_mainmenu();
		
		return "\t\t<script>$result</script>".PHP_EOL;
	}
	
	protected function javascript_mainmenu()
	{
		$items = $this->config->get('mainmenu');
		
		$mainmenu = array();
		
    	foreach($items as $key => $val)
    	{
    		if ($val['type'] === 'item')
    		{
    			$val['code'] = $key;
    			$mainmenu[] = $val;
    		}
    	}
    	
    	$js_mainmenu = json_encode($mainmenu);
    	$js_title = $this->config->get('title');
    	$js_title_sep = $this->config->get('title_separator');
    	$js_title_pos = $this->config->get('title_position');
    	
    	return 'var Website=Website||{};'
    		."Website.mainMenuItems=$js_mainmenu;"
    		."Website.title='$js_title';"
    		."Website.titleSeparator='$js_title_sep';"
    		."Website.titlePosition='$js_title_pos';";
	}
	
	public function config($key){
		return $this->config[$key];
	}
	
}