require 'capistrano/ext/multistage'

server "acts-kenya-db.org", :web, :primary => true

set :application, 'av'
set :stages, %w(production staging)
set :default_stage, 'staging'
set :user, 'actskeny'
set :deploy_via, :remote_cache
set :scm, :git
set :repository, "git@bitbucket.org:paulsschwarz/aquaventures.git"
set :git_enable_submodules, 1
set :branch, 'master'
set :use_sudo, false
set :keep_releases, 3

# before 'deploy:create_symlink' do
#   run "cd #{release_path} && php /usr/local/bin/composer/composer.phar install --prefer-dist"
# end



before "deploy:create_symlink", "deploy:make_dirs"
before "deploy:create_symlink", "deploy:cleanup_dirs"
before "deploy:create_symlink", "deploy:cleanup_files"
before "deploy:create_symlink", "deploy:permissions_webkit"
before "deploy:create_symlink", "deploy:link_rates_config"

namespace :deploy do

  desc "Remove extraneous dirs."
  task :cleanup_dirs do
    run "cd #{latest_release}; rm -Rf .git cookbooks documentation public tmp parent_directory vagrant sql resources;"
  end

  desc "Make required dirs."
  task :make_dirs do
    run "cd #{latest_release}; chgrp actskeny base/application/cache -R; chgrp actskeny base/application/logs -R;"
  end

  desc "Remove extraneous files."
  task :cleanup_files do
    run "cd #{latest_release}; rm -f .gitignore .gitmodules Capfile Vagrantfile README.md;"
  end

  desc "Permission Webkit"
  task :permissions_webkit do
    run "cd #{latest_release}; chmod 777 base/application/cache -R; chmod 777 base/application/logs -R;"
  end

  desc "Remove the deployed rates and symlink the shared rates"
  task :link_rates_config do
    run "cd #{latest_release}; rm base/application/config/rates.php"
    run "ln -s /home1/actskeny/public_html/diveinkenya/rates.php #{latest_release}/base/application/config/rates.php"
  end

end
